package xyz.twovb.sgm.commands.impl;

/*
 * Created by 2vb - 4/6/2024
 */

import dev.rollczi.litecommands.annotations.argument.Arg;
import dev.rollczi.litecommands.annotations.command.Command;
import dev.rollczi.litecommands.annotations.context.Context;
import dev.rollczi.litecommands.annotations.execute.Execute;
import dev.rollczi.litecommands.annotations.permission.Permission;
import org.bukkit.GameMode;
import org.bukkit.World;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import xyz.twovb.sgm.SGM;
import xyz.twovb.sgm.levels.LevelManager;
import xyz.twovb.toolbox.api.CustomPlayer;
import xyz.twovb.toolbox.managers.PlaceholderManager;
import xyz.twovb.toolbox.utils.ChatUtils;

import java.io.IOException;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

@Command(name = "level")
@Permission("sgm.levels")
public class LevelCommand {

    @Execute()
    void execute(@Context CommandSender sender) {
        for (String game : SGM.getInstance().getLevelManager().enabledMaps.keySet()) {
            List<String> maps = SGM.getInstance().getLevelManager().enabledMaps.get(game);
            sender.sendMessage("Game: " + game + ", Maps: " + maps);
        }
    }

    @Execute(name = "create")
    @Permission("sgm.levels.create")
    void create(@Context CommandSender sender, @Arg("game") String game, @Arg("name") Optional<String> LevelName) {
        String name = LevelName.orElse(UUID.randomUUID().toString());
        LevelManager.LevelResult result = SGM.getInstance().getLevelManager().createLevel(name, game);
        switch (result) {
            case SUCCESS:
                sender.sendMessage(ChatUtils.translate(PlaceholderManager.setPlaceholders(SGM.getInstance().getMessages().getString("sgm.level.new"), sender).replace("%level%", name)));
                break;
            case WORLD_EXISTS:
                sender.sendMessage(ChatUtils.translate(PlaceholderManager.setPlaceholders(SGM.getInstance().getMessages().getString("sgm.level.exists"), sender).replace("%level%", name)));
                break;
            case INVALID_ARGS:
                sender.sendMessage(ChatUtils.translate(PlaceholderManager.setPlaceholders(SGM.getInstance().getMessages().getString("sgm.game.not-found"), sender)));
                break;
            case UNKNOWN:
            default:
                sender.sendMessage(ChatUtils.translate(PlaceholderManager.setPlaceholders(SGM.getInstance().getMessages().getString("commands.unknown-error"), sender).replace("%error%", "(UNKNOWN)")));
                break;
        }
    }

    @Execute(name = "edit")
    @Permission("sgm.levels.edit")
    void edit(@Context Player player, @Arg("name") String name) {
        CustomPlayer cPlayer = new CustomPlayer(player);
        World world = LevelManager.getLevel(name);
        player.teleport(world.getSpawnLocation());
        player.setGameMode(GameMode.CREATIVE);
        cPlayer.sendMessage("<red>tele&aport");
    }

    @Execute(name = "export")
    @Permission("sgm.levels.export")
    void export(@Context CommandSender sender, @Arg("name") String name) {
        try {
            SGM.getInstance().getLevelManager().exportLevel(name);
            sender.sendMessage(ChatUtils.translate(PlaceholderManager.setPlaceholders(SGM.getInstance().getMessages().getString("sgm.level.export.done"), sender).replace("%level%", name)));
        } catch (IOException e) {
            sender.sendMessage(ChatUtils.translate(PlaceholderManager.setPlaceholders(SGM.getInstance().getMessages().getString("sgm.level.not-found"), sender)));
        }
    }

}
